Package: srcpkgs
Title: R Source Packages Manager
Version: 0.2.1
Authors@R: 
    person("Karl", "Forner", , "karl.forner@gmail.com", role = c("aut", "cre", "cph"))
Description: Manage a collection/library of R source packages. Discover, document, load, test 
  source packages. Enable to use those packages as if they were actually installed. Quickly reload
  only what is needed on source code change. Run tests and checks in parallel.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/kforner/srcpkgs
BugReports: https://github.com/kforner/srcpkgs/issues
Imports: cli, clitable, devtools, pkgload, testthat, stats, utils
Suggests: knitr, rmarkdown, withr
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-20 11:01:33 UTC; vscode
Author: Karl Forner [aut, cre, cph]
Maintainer: Karl Forner <karl.forner@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-20 13:00:02 UTC
Built: R 4.5.2; ; 2026-04-20 15:06:55 UTC; unix
