Package: elves
Title: elves dummy package for scrpkgs
Description: elves dummy package for scrpkgs.
Version: 0.1
Authors@R: person("Karl", "Forner", , "karl.forner@gmail.com", role = c("aut", "cre"))
Imports: galadriel, legolas
License: MIT
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Suggests: testthat
