# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,pkg_check)
S3method(as.data.frame,pkg_test)
S3method(as.data.frame,pkgs_check)
S3method(as.data.frame,pkgs_test)
S3method(as.data.frame,srcpkgs)
S3method(as.logical,pkg_check)
S3method(as.logical,pkg_test)
S3method(as.logical,pkgs_check)
S3method(as.logical,pkgs_test)
S3method(print,pkg_test)
S3method(print,pkgs_check)
S3method(print,pkgs_test)
S3method(print,srcpkg)
S3method(print,srcpkgs)
S3method(summary,pkg_check)
S3method(summary,pkg_test)
S3method(summary,pkgs_check)
S3method(summary,pkgs_test)
export(find_srcpkgs)
export(get_srcpkgs)
export(hack_r_loaders)
export(pkg_check)
export(pkg_list_attached)
export(pkg_load)
export(pkg_roxygenise)
export(pkg_test)
export(pkg_unload)
export(pkgs_check)
export(pkgs_deps)
export(pkgs_install)
export(pkgs_test)
export(reset)
export(settings)
export(setup_and_get_dummy_srcpkg)
export(srcpkgs)
export(unhack_r_loaders)
importFrom(cli,cli_text)
importFrom(clitable,cli_table)
importFrom(testthat,test_dir)
importFrom(utils,getFromNamespace)
