#' @section Features:
#'
#' `srcpkgs` main objective is to ease development on any project
#' that uses a collection of R source packages (a library).
#' It is able to figure out which dependencies are source packages, and is able
#' to quickly detect changes in any of the used source packages.
#'

#'
"_PACKAGE"

NULL
