% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{legolas}
\alias{legolas}
\title{provides legolas}
\usage{
legolas()
}
\value{
as a list
}
\description{
provides legolas
}
