% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{hobbits}
\alias{hobbits}
\title{list the available hobbits}
\usage{
hobbits()
}
\value{
hobbit names as character
}
\description{
list the available hobbits
}
