% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{lotr}
\alias{lotr}
\title{list all characters}
\usage{
lotr()
}
\value{
as a list
}
\description{
list all characters
}
