% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{aragorn}
\alias{aragorn}
\title{provides aragorn}
\usage{
aragorn()
}
\value{
as a list
}
\description{
provides aragorn
}
