% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg_utils.R
\name{pkg_create}
\alias{pkg_create}
\title{creates and populates a R package-like folder programmatically, useful for writing tests}
\usage{
pkg_create(
  dir,
  name,
  functions = list(dummy = function() "DUMMY"),
  imports = NULL,
  depends = NULL,
  suggests = NULL,
  namespace = FALSE,
  roxygen_imports = FALSE,
  ignore_hidden_files = TRUE
)
}
\arguments{
\item{dir}{the directory in which to create the package, as a string}

\item{name}{the package name, as a string}

\item{functions}{a named list of functions to add to the package}

\item{imports}{the "imports" dependencies}

\item{depends}{the "depends" dependencies}

\item{suggests}{the "suggests" dependencies}

\item{namespace}{whether to write the namespace file (currently only applicable to the imports.
N.B: if the namespace file is generated, roxygen will refuse to update it}

\item{roxygen_imports}{whether to write the roxygen statements to defined the imports}

\item{ignore_hidden_files}{whether to create a \code{.Rbuildignore} file to ignore hidden files.}
}
\value{
the srcpkg instance, invisibly
}
\description{
basically a wrapper around \code{utils::package.skeleton()}
}
\keyword{internal}
