% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{elrond}
\alias{elrond}
\title{provides elrond}
\usage{
elrond()
}
\value{
as a list
}
\description{
provides elrond
}
