% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{galadriel}
\alias{galadriel}
\title{provides galadriel}
\usage{
galadriel()
}
\value{
as a list
}
\description{
provides galadriel
}
