% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{gimli}
\alias{gimli}
\title{provides gimli}
\usage{
gimli()
}
\value{
as a list
}
\description{
provides gimli
}
