% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{frodo}
\alias{frodo}
\title{provides frodo}
\usage{
frodo()
}
\value{
as a list
}
\description{
provides frodo
}
