% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{bilbo}
\alias{bilbo}
\title{provides bilbo}
\usage{
bilbo()
}
\value{
as a list
}
\description{
provides bilbo
}
