% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{gandalf}
\alias{gandalf}
\title{provides gandalf}
\usage{
gandalf()
}
\value{
as a list
}
\description{
provides gandalf
}
